"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.parsePathInfo = exports.upload = exports.download = exports.copy = exports.list = void 0;
const cloud_bucket_1 = require("cloud-bucket");
const fs_extra_1 = require("fs-extra");
const utils_1 = require("./utils");
// --------- Public list/copy/del/download/upload --------- //
async function list(pathInfo) {
    const bucket = await getBucketFromPathInfo(pathInfo);
    const result = await bucket.listFiles(pathInfo.path);
    return result;
}
exports.list = list;
async function copy(from, to) {
    const files = await list(from);
    const destBucket = await getBucketFromPathInfo(to);
    // FIXME: Need to be implemented
}
exports.copy = copy;
async function download(from, destDir) {
    if (from.path == null) {
        throw new Error(`Cannot download - undefined remote path: ${from.store}:${from.path}`);
    }
    const bucket = await getBucketFromPathInfo(from);
    return bucket.download(from.path, destDir);
}
exports.download = download;
async function upload(localFile, dest) {
    if (dest.path == null) {
        throw new Error(`Cannot download - undefined remote path: ${dest.store}:${dest.path}`);
    }
    const bucket = await getBucketFromPathInfo(dest);
    return bucket.upload(localFile, dest.path);
}
exports.upload = upload;
// --------- /Public list/cp/del/download/upload ------, --- //
// --------- public PathInfo --------- //
const formatInfo = `'storename:path' or just 'storename'`;
function parsePathInfo(storeAndPathStr) {
    if (storeAndPathStr == null || storeAndPathStr.length == 0) {
        throw new Error(`gs need to have a path like ${formatInfo}`);
    }
    const storeAndPath = storeAndPathStr.split(':');
    if (storeAndPath.length > 2) {
        throw new Error(`Path ${storeAndPathStr} is invalid, needs to be of format like  ${formatInfo}`);
    }
    const [store, path] = storeAndPath;
    return { store, path };
}
exports.parsePathInfo = parsePathInfo;
// --------- /public PathInfo --------- //
// --------- Private Utils --------- //
async function getBucketFromPathInfo(pathInfo) {
    const bucketCfgs = await loadRawBucketsConfig();
    const rawCfg = bucketCfgs === null || bucketCfgs === void 0 ? void 0 : bucketCfgs[pathInfo.store];
    if (rawCfg == null) {
        throw new Error('VDEV ERROR - cannot find bucket ${pathInfo.store} in .vdev-buckets.yaml file');
    }
    return cloud_bucket_1.getBucket({ ...rawCfg, log: true });
}
async function loadRawBucketsConfig() {
    const files = ['.vdev-buckets.yaml', 'vdev-buckets.yaml'];
    let vdevBuckets;
    for (const file of files) {
        if (await fs_extra_1.pathExists(file)) {
            vdevBuckets = await utils_1.loadYaml(file);
            break;
        }
    }
    if (vdevBuckets != null) {
        const confs = vdevBuckets.buckets;
        // add the .name to each bucket info
        for (let name in confs) {
            confs[name].name = name;
        }
        return confs;
    }
}
// --------- /Private Utils --------- //
