"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.dpush_image_ex = exports.dpush_prep = exports.realm_init = void 0;
const p_spawn_1 = require("p-spawn");
const block_1 = require("./block");
const main_1 = require("./main");
//#region    ---------- Hook Functions ---------- 
// finish initializing a realm object
async function realm_init(realm) {
    // a aws realm must have a profile. Default is 'default'
    if (!realm.profile) {
        realm.profile = 'default';
    }
}
exports.realm_init = realm_init;
async function dpush_prep(realm, serviceNames) {
    const blockByName = await block_1.loadBlocks();
    const repoNames = serviceNames.map(sn => main_1.getRepositoryName(blockByName[sn]));
    const existingRepositoryNames = await getAwsExistingRepositoryNames(realm);
    const missingRepositoryNames = repoNames.filter(n => !existingRepositoryNames.includes(n));
    await createRepositories(realm, missingRepositoryNames);
}
exports.dpush_prep = dpush_prep;
async function dpush_image_ex(realm, ex, remoteImage) {
    console.log(`dpush - recovering - first docker dpush to ${remoteImage} failed, trying to recover`);
    // AWS CLI 2
    try {
        // get ecr password
        const cmdGetPwd = await p_spawn_1.spawn('aws', ['ecr', 'get-login-password', '--profile', realm.profile], { capture: 'stdout' });
        const pwd = cmdGetPwd.stdout.trim();
        // docker login (leave output to console)
        const cmdDockerLogin = await p_spawn_1.spawn('docker', ['login', '--username', 'AWS', '--password-stdin', realm.registry], { input: pwd });
        console.log('dpush - relogin success');
        console.log(`dpush - trying to push again ${remoteImage}`);
        await p_spawn_1.spawn('docker', ['push', remoteImage]);
        console.log('dpush - fully recovered');
    }
    catch (ex) {
        console.log('dpush - error - relogin/repush failed', ex);
        throw ex;
    }
    // AWS CLI 1.x
    // aws ecr get-login --no-include-email
    // const reloginCmd = await spawn('aws', ['ecr', 'get-login', '--no-include-email', '--profile', realm.profile], { capture: 'stdout' });
    // const reloginArg = reloginCmd.stdout!.trim().split(' ');
    // reloginArg.shift(); // remove the 'docker' first item.
    // try {
    // 	const relogin = await spawn('docker', reloginArg, { capture: 'stdout' });
    // 	console.log('dpush - relogin success');
    // 	console.log(`dpush - trying to push again ${remoteImage}`);
    // 	await spawn('docker', ['push', remoteImage]);
    // 	console.log('dpush - fully recovered');
    // } catch (ex2) {
    // 	console.log('dpush - error - relogin/repush failed', ex2);
    // 	throw ex2;
    // }
    return true;
}
exports.dpush_image_ex = dpush_image_ex;
//#endregion ---------- /Hook Functions ---------- 
//#region    ---------- Utils ---------- 
async function createRepositories(realm, repoNames) {
    for (const repo of repoNames) {
        // aws ecr create-repository --profile default --repository-name cstar-agent
        console.log(`Info - aws ECR repository ${repo} does not exist, creating...`);
        await p_spawn_1.spawn('aws', ['ecr', 'create-repository', '--profile', realm.profile, '--repository-name', repo]);
    }
}
async function getAwsExistingRepositoryNames(realm) {
    // NOTE: Somehow even when the login has expired, this call works (it's the push that does not work)
    const dataStr = await p_spawn_1.spawn('aws', ['ecr', 'describe-repositories', '--profile', realm.profile], { capture: 'stdout' });
    const repositories = JSON.parse(dataStr.stdout.trim()).repositories;
    if (repositories) {
        return repositories.map((r) => r.repositoryName);
    }
    else {
        return [];
    }
}
//#endregion ---------- /Utils ----------
